package tictactoe.bThreads;
/**
 * A bthread that only tracks the moves and displays the board. 
 * No clicks on it. 
 * Design to work as an updated GUI including back tracking.  
 */

import static bp.BProgram.*;
import static bp.eventSets.EventSetConstants.*;
import static tictactoe.events.StaticEvents.*;

import java.awt.*;

import javax.swing.*;

import tictactoe.events.Move;
import tictactoe.externalApp.TicTacToe;
import bp.BThread;
import bp.eventSets.EventSet;
import bp.eventSets.EventsOfClass;
import bp.exceptions.BPJException;


/**
 * BThread for updating the labels of the buttons.
 */
@SuppressWarnings("serial")
public class TrackBoard extends BThread {
	
	private JFrame window = new JFrame("Game Tracker");
	JButton buttons[][] = new JButton[3][];
	JLabel message = new JLabel();
	EventSet boardEvents  = new EventSet(XWin, OWin, draw, new EventsOfClass(Move.class)); 
	String msg; 
	
//--------------------------------------
	public void runBThread() throws BPJException {
		interruptingEvents = new EventSet(gameOver);
		// initialize window  
		window.setSize(150, 150);
		window.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		window.setLayout(new BorderLayout());
		window.setLocation(new Point(400,400));
		

		// The board
		JPanel board = new JPanel();
		board.setLayout(new GridLayout(3, 3));
		// The message label
		message.setHorizontalAlignment(JLabel.CENTER);

		// Create buttons
		for (int i = 0; i < 3; i++) {
			buttons[i] = new JButton[3];
			for (int j = 0; j < 3; j++) {
				buttons[i][j] = new TrackButton(i, j);
				board.add(buttons[i][j]);
				buttons[i][j].setEnabled(false); 

				// Dont add action listener - this is passive 
// 				buttons[i][j].addActionListener(this); 
			}
		}
		// Add the board and the message component to the window
		window.add(board, BorderLayout.CENTER);
		window.add(message, BorderLayout.SOUTH);

		// Make the window visible
		window.setVisible(true);

		 


// TODO - need to update the state based on the observed events.  

		while (true) {

			// Wait for an event
			bp.bSync(none, boardEvents, none);
			
			if (bp.lastEvent instanceof Move) { 
				// Update the board with the move
				Move move = (Move) bp.lastEvent;
				JButton btt = buttons[move.row][move.col];
				btt.setText(move.displayString());
			}
			else { 
			if (bp.lastEvent == XWin) {
				msg = "X Wins";
			} else if (bp.lastEvent == OWin) {
				msg = "O Wins";
			} else 
				msg = "A Draw"; 
			message.setText(msg);
			} 
			
		
	}
} 
		


/**
 * A button that remembers its position on the board
 */
@SuppressWarnings("serial")
class TrackButton extends JButton {
	int row;
	int col;

	/**
	 * Constructor.
	 *
	 * @param row
	 *            The row of the button.
	 * @param col
	 *            The column of the button.
	 */
	public TrackButton(int row, int col) {
		super();
		this.row = row;
		this.col = col;
	}

}
}

